@@ -38,7 +38,7 @@ from kiwi.datatypes import converter, number, Decimal
 from kiwi.currency import currency # after datatypes
 from kiwi.enums import Alignment
 from kiwi.log import Logger
-from kiwi.python import slicerange
+from kiwi.python import enum, slicerange
 from kiwi.utils import PropertyObject, gsignal, gproperty, type_register
 
 _ = lambda m: gettext.dgettext('kiwi', m)
@@ -69,6 +69,9 @@ class Column(PropertyObject, gobject.GObject):
           the attribute
       - B{data-type}: object I{str}
         - the type of the attribute that will be inserted into the column.
+          Supported data types: bool, int, float, str, unicode,
+          decimal.Decimal, datetime.date, datetime.time, datetime.datetime,
+          gtk.gdk.Pixbuf, L{kiwi.currency.currency}, L{kiwi.python.enum}.
       - B{visible}: bool I{True}
         - specifying if it is initially hidden or shown.
       - B{justify}: gtk.Justification I{None}
@@ -123,6 +126,7 @@ class Column(PropertyObject, gobject.GObject):
       - B{use_markup}: bool I{False}
         - If true, the text will be rendered with markup
     """
+    __gtype_name__ = 'Column'
     gproperty('title', str)
     gproperty('data-type', object)
     gproperty('visible', bool, default=True)
@@ -236,13 +240,16 @@ class Column(PropertyObject, gobject.GObject):
 
     def as_string(self, data):
         data_type = self.data_type
-        if (self.format or
+        if data is None:
+            text = ''
+        elif (self.format or
             data_type == float or
             data_type == Decimal or
             data_type == currency or
             data_type == datetime.date or
             data_type == datetime.datetime or
-            data_type == datetime.time):
+            data_type == datetime.time or
+            issubclass(data_type, enum)):
             conv = converter.get_converter(data_type)
             text = conv.as_string(data, format=self.format or None)
         elif self.format_func:
@@ -443,6 +450,10 @@ class ObjectList(PropertyObject, gtk.ScrolledWindow):
       - B{row-activated} (object):
         - Emitted when a row is "activated", eg double clicked or pressing
           enter. See the GtkTreeView documentation for more information
+      - B{selection-changed} (object):
+        - Emitted when the selection changes for the ObjectList
+          enter. See the documentation on GtkTreeSelection::changed
+          for more information
       - B{double-click} (object):
         - Emitted when a row is double-clicked, mostly you want to use
           the row-activated signal instead to be able catch keyboard events.
@@ -588,10 +599,7 @@ class ObjectList(PropertyObject, gtk.ScrolledWindow):
 
     def __contains__(self, instance):
         "item in list"
-        for row in self._model:
-            if row[COL_MODEL] == instance:
-                return True
-        return False
+        return bool(self._iters.get(id(instance), False))
 
     def __iter__(self):
         "for item in list"
@@ -627,7 +635,15 @@ class ObjectList(PropertyObject, gtk.ScrolledWindow):
     def __setitem__(self, arg, item):
         "list[n] = m"
         if isinstance(arg, (int, gtk.TreeIter, str)):
-            self._model[arg] = (item,)
+            model = self._model
+            olditem = model[arg][COL_MODEL]
+            model[arg] = (item,)
+
+            # Update iterator cache
+            iters = self._iters
+            iters[id(item)] = model[arg].iter
+            del iters[id(olditem)]
+
         elif isinstance(arg, slice):
             raise NotImplementedError("slices for list are not implemented")
         else:
@@ -776,12 +792,6 @@ class ObjectList(PropertyObject, gtk.ScrolledWindow):
         sorted = None
         expand = False
         for column in columns:
-            if column.searchable:
-                if searchable:
-                    raise ValueError("Can't make column %s searchable, column"
-                                     " %s is already set as searchable" % (
-                        column.attribute, searchable.attribute))
-                searchable = column.searchable
             if column.sorted:
                 if sorted:
                     raise ValueError("Can't make column %s sorted, column"
@@ -834,6 +844,8 @@ class ObjectList(PropertyObject, gtk.ScrolledWindow):
 
         if column.use_stock:
             cell_data_func = self._cell_data_pixbuf_func
+        elif issubclass(column.data_type, enum):
+            cell_data_func = self._cell_data_combo_func
         else:
             cell_data_func = self._cell_data_text_func
 
@@ -930,6 +942,25 @@ class ObjectList(PropertyObject, gtk.ScrolledWindow):
             prop = 'pixbuf'
             if column.editable:
                 raise TypeError("use-stock columns cannot be editable")
+        elif issubclass(data_type, enum):
+            if data_type is enum:
+                raise TypeError("data_type must be a subclass of enum")
+
+            model = gtk.ListStore(str, object)
+            items = data_type.names.items()
+            items.sort()
+            for key, value in items:
+                model.append((key.lower().capitalize(), value))
+
+            renderer =  gtk.CellRendererCombo()
+            renderer.set_property('model', model)
+            renderer.set_property('text-column', 0)
+            renderer.set_property('has-entry', True)
+            if column.editable:
+                renderer.set_property('editable', True)
+                renderer.connect('edited', self._on_renderer_combo__edited,
+                                 self._model, column.attribute, column)
+            prop = 'model'
         elif issubclass(data_type, (datetime.date, datetime.time,
                                     basestring, number,
                                     currency)):
@@ -1007,10 +1038,10 @@ class ObjectList(PropertyObject, gtk.ScrolledWindow):
 
     # TreeView
     def _treeview_search_equal_func(self, model, tree_column, key, treeiter, column):
-        "for searching inside the treeview"
+        "for searching inside the treeview, case-insensitive by default"
         data = column.get_attribute(model[treeiter][COL_MODEL],
                                     column.attribute, None)
-        if data.startswith(key):
+        if data.lower().startswith(key.lower()):
             return False
         return True
 
@@ -1097,6 +1128,14 @@ class ObjectList(PropertyObject, gtk.ScrolledWindow):
         pixbuf = self.render_icon(data, column.icon_size)
         renderer.set_property(renderer_prop, pixbuf)
 
+    def _cell_data_combo_func(self, tree_column, renderer, model, treeiter,
+                              (column, renderer_prop)):
+        row = model[treeiter]
+        data = column.get_attribute(row[COL_MODEL],
+                                    column.attribute, None)
+        text = column.as_string(data)
+        renderer.set_property('text', text.lower().capitalize())
+
     def _on_renderer__toggled(self, renderer, path, column):
         setattr(self._model[path][COL_MODEL], column.attribute,
                 not renderer.get_active())
@@ -1143,6 +1182,22 @@ class ObjectList(PropertyObject, gtk.ScrolledWindow):
         setattr(obj, attr, value)
         self.emit('cell-edited', obj, attr)
 
+    def _on_renderer_combo__edited(self, renderer, path, text,
+                                  model, attr, column):
+        obj = model[path][COL_MODEL]
+        if not text:
+            return
+
+        value_model = renderer.get_property('model')
+        for row in value_model:
+            if row[0] == text:
+                value = row[1]
+                break
+        else:
+            raise AssertionError
+        setattr(obj, attr, value)
+        self.emit('cell-edited', obj, attr)
+
     def _on_renderer__edited(self, renderer, path, value, column):
         data_type = column.data_type
         if data_type in number:
@@ -1547,13 +1602,31 @@ class ObjectList(PropertyObject, gtk.ScrolledWindow):
         """
         self._treeview.set_headers_visible(value)
 
+    def set_visible_rows(self, rows):
+        """
+        Sets the number of visible rows of the treeview. This is useful to use
+        instead of set_size_request() directly, since you can avoid using raw
+        pixel sizes.
+        @param rows: number of rows to show
+        """
+
+        treeview = self._treeview
+        if treeview.get_headers_visible():
+            treeview.realize()
+            header_h = self._get_header_height()
+        else:
+            header_h = 0
+
+        column = treeview.get_columns()[0]
+        h = column.cell_get_size()[-1]
+
+        focus_padding = treeview.style_get_property('focus-line-width') * 2
+        treeview.set_size_request(-1, header_h + (rows * (h + focus_padding)))
+
 type_register(ObjectList)
 
-class ObjectRow(object):
-    def __init__(self, iter):
-        self.iter = iter
-
 class ObjectTree(ObjectList):
+    __gtype_name__ = 'ObjectTree'
     def __init__(self, columns=[], objects=None, mode=gtk.SELECTION_BROWSE,
                  sortable=False, model=None):
         if not model:
@@ -1561,17 +1634,18 @@ class ObjectTree(ObjectList):
         ObjectList.__init__(self, columns, objects, mode, sortable, model)
 
     def _append_internal(self, parent, instance, select, prepend):
-        if parent is not None and not isinstance(parent, ObjectRow):
-            raise TypeError("parent must be an ObjectRow or None")
+        iters = self._iters
+        parent_id = id(parent)
+        if parent_id in iters:
+            parent_iter = iters[parent_id]
+        elif parent is None:
+            parent_iter = None
+        else:
+            raise TypeError("parent must be an Object, ObjectRow or None")
 
         # Freeze and save original selection mode to avoid blinking
         self._treeview.freeze_notify()
 
-        if parent:
-            parent_iter = parent.iter
-        else:
-            parent_iter = None
-
         if prepend:
             row_iter = self._model.prepend(parent_iter, (instance,))
         else:
@@ -1586,24 +1660,56 @@ class ObjectTree(ObjectList):
             self._select_and_focus_row(row_iter)
         self._treeview.thaw_notify()
 
-        return ObjectRow(row_iter)
+        return instance
 
     def append(self, parent, instance, select=False):
         """
-        @param parent: ObjectRow of the parent
+        @param parent: Object or None, representing the parent
         @param instance: the instance to be added
         @param select: select the row
+        @returns: the appended object
         """
         return self._append_internal(parent, instance, select, prepend=False)
 
     def prepend(self, parent, instance, select=False):
         """
-        @param parent: ObjectRow of the parent
+        @param parent: Object or None, representing the parent
         @param instance: the instance to be added
         @param select: select the row
+        @returns: the prepended object
         """
         return self._append_internal(parent, instance, select, prepend=True)
 
+    def expand(self, instance, open_all=True):
+        """
+        This method opens the row specified by path so its children
+        are visible.
+        @param instance: an instance to expand at
+        @param open_all: If True, expand all rows, otherwise just the
+        immediate children
+        """
+        objid = id(instance)
+        if not objid in self._iters:
+            raise ValueError("instance %r is not in the list" % instance)
+        treeiter = self._iters[objid]
+
+        self.get_treeview().expand_row(
+            self._model[treeiter].path, open_all)
+
+    def collapse(self, instance):
+        """
+        This method collapses the row specified by path
+        (hides its child rows, if they exist).
+        @param instance: an instance to collapse
+        """
+        objid = id(instance)
+        if not objid in self._iters:
+            raise ValueError("instance %r is not in the list" % instance)
+        treeiter = self._iters[objid]
+
+        self.get_treeview().collapse_row(
+            self._model[treeiter].path)
+
 type_register(ObjectTree)
 
 class ListLabel(gtk.HBox):
@@ -1736,7 +1842,7 @@ class SummaryLabel(ListLabel):
         attr = column.attribute
         get_attribute = column.get_attribute
 
-        value = sum([get_attribute(obj, attr) for obj in self._klist],
+        value = sum([get_attribute(obj, attr, 0) for obj in self._klist],
                     column.data_type('0'))
 
         self.set_value(column.as_string(value))
